# -*- coding: utf8 -*-
from __future__ import unicode_literals

import six
import errno
import socket
import datetime
import time
import threading
from driverng.transport.base import BaseTransport, TransportException
if six.PY3:
    from http.server import BaseHTTPRequestHandler, HTTPServer
else:
    from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

filename_received = {}
filename_waiting = {}
client_reponses = {}


class HTTPHandler(BaseHTTPRequestHandler):

    def get_filename(self):
        return "resu_c_aqua-%s-%s.xml" % (self.client_address[1], datetime.datetime.now().strftime("%Y%m%d%H%M%S"))

    def _set_headers(self):
        self.send_response(200)
        self.send_header('Content-type', 'text/xml')
        #  self.end_headers() not supported by Aqua client!

    def do_GET(self):
        self._set_headers()
        self.wfile.write("<html><body><h1>Get not implemented!</h1></body></html>")

    def do_HEAD(self):
        self._set_headers()

    def do_POST(self):
        content_length = int(self.headers['Content-Length'])  # <--- Gets the size of data
        post_data = self.rfile.read(content_length)  # <--- Gets the data itself
        filename = self.get_filename()
        filename_received[filename] = (post_data, self)

        self._set_headers()
        for i in range(5):  # 5s pour recevoir la reponse
            reponse = client_reponses.get(self, None)
            if reponse:
                self.wfile.write(reponse)
                del client_reponses[self]
                return
            else:
                time.sleep(1)

    def finish(self):
        if client_reponses.get(self, None):
            del client_reponses[self]


class MyHTTPServer(HTTPServer):

    request_queue_size = 20  # nombre max de clients en attente


class HTTPTransport(BaseTransport):

    def __init__(self, host=None, port=None, mode="server", **kwargs):
        super(HTTPTransport, self).__init__(**kwargs)

        if host is None:
            raise TransportException("L'adresse ip et le port http ne sont pas paramétrés sur la connexion")
        if port is None:
            host, port = host.rsplit(":", 1)
            port = int(port)

        if mode == "server":
            self.httpd = MyHTTPServer((host, port), HTTPHandler)
            self.logger.info('Starting httpd...')
            t = threading.Thread(target=self.httpd.serve_forever)
            t.daemon = True
            t.start()
        else:  # client

            self.logger.info("Try to connect to %s:%d", host, port)
            raise Exception('transport Http client non-implémenté')

        self.logger.info('Starting httpd OK')

    def set_timeout(self, timeout):
        pass

    def poll(self, timeout, wqueue):
        if filename_received:
            return True
        return False

    def pre_send(self, meta=None):
        self.logger.info("pre_send meta:%s", meta)
        self.meta = meta

    def _read(self, size):
        filename, value = filename_received.items()[0]
        self.filename = filename
        self.client = value[1]
        del filename_received[filename]
        filename_waiting[filename] = value
        client_reponses.setdefault(self.client, None)

        return value[0]

    def get_recv_filename(self):
        return self.filename

    def _write(self, data):
        filename = self.meta.get('filename', None)
        if not filename:  # reponse automatique à la derniere trame
            client_reponses[self.client] = data
            return
        values = filename_waiting.get(filename, None)
        if values:
            client_reponses[values[1]] = data  # envoie au bon client
            del filename_waiting[filename]
        else:
            self.logger.error('httpd _write, no handler client found')
        return

    def close(self):
        try:
            self.httpd.shutdown()
            self.httpd.close()
        except socket.error as e:
            if e.args[0] not in (errno.ENOTCONN, errno.EBADF):
                raise
